import numpy as np
import pandas as pd
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit import Chem
from rdkit import DataStructs
import matplotlib.pyplot as plt
from sklearn.manifold import TSNE
import re


#read molecules information
chart=open('Mols.txt','r',encoding='utf-8') #The first column in Mols.txt is the name of the molecule, the second column is the SMILES formula, no table header
smilesDict={i.split('\t')[0]:i.split('\t')[1].strip() for i in chart}
molDict = {name: Chem.MolFromSmiles(smiles) for name, smiles in smilesDict.items()} 
mols = [mol for mol in molDict.values()]
nms = [nm for nm in molDict.keys()]
chart.close()

#Extracting Morgan fingerprints
fpDict = {name: AllChem.GetMorganFingerprintAsBitVect(mol, 2, nBits=1024) for name, mol in molDict.items()}
fps = [fp for fp in fpDict.values()]

#Caculate similarity
nw=open('network.txt','w')
cd=[]
names=list(fpDict.keys())
for nameA, fpA in fpDict.items():
    names.remove(nameA)
    for nameB in names:
        Dice = DataStructs.DiceSimilarity(fpA, fpDict[nameB])
        if Dice>=0.45:
            nw.write('{}\t{}\t{:.2}\n'.format(nameA,nameB,Dice))
            cd.append(nameA)
            cd.append(nameB)
            print(nameA)
nw.close()
cd=set(cd)


#t-SNE
def fp2arr(fp):
    from rdkit import DataStructs
    arr = np.zeros((1,))
    DataStructs.ConvertToNumpyArray(fp, arr)
    return arr
fpMtx = np.array([fp2arr(fp) for fp in fps])
tsne = TSNE(n_components=2)
res = tsne.fit_transform(fpMtx)
t=res.T
np.savetxt('coordinate.txt',res)

#load coordinate
cdnt=np.loadtxt('coordinate.txt')
cdnt_dict={}
n=0
for i in cdnt:
    cdnt_dict[n]=i
    n+=1

#plot
plt.figure(figsize=(10,10))
x=0
for i in cd:
    x+=1
    plt.scatter(cdnt[int(i)-1][0], cdnt[int(i)-1][1],s=100,c='tomato',edgecolors='red',alpha=0.2)
#for i in range(n):
    #plt.annotate(i+1, xy = (cdnt[i][0], cdnt[i][1]), xytext = (cdnt[i][0], cdnt[i][1]),fontsize=10)
edges=open('network.txt','r').readlines()
for i in edges:
    x1=cdnt_dict[int(i.split('\t')[0])-1][0]
    x2=cdnt_dict[int(i.split('\t')[1])-1][0]
    y1=cdnt_dict[int(i.split('\t')[0])-1][1]
    y2=cdnt_dict[int(i.split('\t')[1])-1][1]
    plt.plot([x1,x2],[y1,y2], color='lightskyblue',linewidth=1.0,alpha=0.2) 